import os

from django.conf import settings
from django.http import HttpResponse
from django.utils.deprecation import MiddlewareMixin

from babel_transpiling.utils import get_options, get_absolute_path, get_file_content, get_transpiler


class StaticFilesTranspilingMiddleware(MiddlewareMixin):
    """
    Transpiling static files

    Note: if you use django build-in `runserver` command?
        You **MUST** add `--nostatic` option to command to make sure static file request also pass through middleware
    """
    def __init__(self, get_response=None):
        super().__init__(get_response)
        self.options = get_options()
        if settings.DEBUG:
            self.ctx = get_transpiler(self.options)

    def process_response(self, request, response):
        if not self.check_request(request):
            return response

        if settings.DEBUG:
            return self.transpiling(request, response)
        return self.fixup_content_type(request, response)

    def check_request(self, request):
        if not request.path.startswith(settings.STATIC_URL):
            return False

        file_path = request.path[len(settings.STATIC_URL):]
        _, file_suffix = os.path.splitext(file_path)

        if file_suffix not in self.options['extensions']:
            return False

        return True

    def transpiling(self, request, response):
        path = get_absolute_path(request.path[len(settings.STATIC_URL):])
        if not path:
            return response

        result = self.ctx.call('Babel.transform', get_file_content(path), self.options['options'])
        _, file_suffix = os.path.splitext(path)
        return HttpResponse(content=result['code'], content_type=self.options['mimetypes'][file_suffix])

    def fixup_content_type(self, request, response):
        if response.status_code != 200:
            return response
        _, file_suffix = os.path.splitext(request.path)
        response['Content-Type'] = self.options['mimetypes'][file_suffix]
        return response



import os

from django.contrib.staticfiles.storage import StaticFilesStorage

from babel_transpiling.utils import get_options, get_transpiler


class StaticFilesTranspilingStorage(StaticFilesStorage):
    """
    Transpiling static files when execute `collectstatic` command
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.options = get_options()
        self.ctx = get_transpiler(self.options)

    def _save(self, name, content):
        origin = super()._save(name, content)
        _, file_suffix = os.path.splitext(name)

        if file_suffix not in self.options['extensions']:
            return origin

        file = open(self.path(name), 'r+')
        result = self.ctx.call('Babel.transform', file.read(), self.options['options'])
        file.seek(0)
        file.truncate()
        file.write(result['code'])
        file.close()
        return origin



import posixpath

from py_mini_racer import py_mini_racer
from django.conf import settings
from django.contrib.staticfiles import finders
from django.core.exceptions import ImproperlyConfigured


def get_options():
    default = {
        'transpiler': 'npm/@babel/standalone@7.12.15/babel.min.js',
        'extensions': ['.jsx'],
        'options': {
            "presets": ["react"],
            "generatorOpts": {
                "jsescOption": {
                    "minimal": True
                }
            }
        },
        'mimetypes': {
            '.jsx': 'application/javascript'
        },
        'setup': None
    }

    if hasattr(settings, 'BABEL_TRANSPILING'):
        default.update(settings.BABEL_TRANSPILING)

    path = get_absolute_path(default['transpiler'])
    if not path:
        raise ImproperlyConfigured('BABEL_TRANSPILING.transpiler')

    if default['setup']:
        path = get_absolute_path(default['setup'])
        if not path:
            raise ImproperlyConfigured('BABEL_TRANSPILING.setup')

    mimetype_keys = list(default['mimetypes'].keys())
    for item in default['extensions']:
        if item not in mimetype_keys:
            raise ImproperlyConfigured(f'BABEL_TRANSPILING.mimetypes missing extension: {item}')

    return default


def get_absolute_path(path):
    normalized_path = posixpath.normpath(path).lstrip('/')
    return finders.find(normalized_path)


def get_file_content(path):
    file = open(path)
    code = file.read()
    file.close()
    return code


def get_transpiler(options):
    ctx = py_mini_racer.MiniRacer()
    ctx.eval(get_file_content(get_absolute_path(options['transpiler'])))

    if options['setup']:
        ctx.eval(get_file_content(get_absolute_path(options['setup'])))

    return ctx



#!/usr/bin/env python3
import os
import re
import sys

from setuptools import find_packages, setup

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 6)

# This check and everything above must remain compatible with Python.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
Requires Python {}.{}, but you're trying
to install it on Python {}.{}.
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)


def read(f):
    return open(f, 'r', encoding='utf-8').read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('babel_transpiling')

setup(
    name='django-babel_transpiling',
    version=version,
    url='https://github.com/DistPub/django-babel-transpiling',
    license='MIT',
    description='Bring babel to your django project, transpiling static .jsx files on the fly, without NodeJS require!',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='Smite Chow',
    author_email='xiaopengyou2no1@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['whitenoise', 'py_mini_racer'],
    python_requires=">=3.6",
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
    ],
)
